@@ -20,7 +20,17 @@ import static com.carrotsearch.randomizedtesting.RandomizedTest.atMost;
 
 import java.io.IOException;
 import java.util.Arrays;
-import org.apache.lucene.document.*;
+import org.apache.lucene.document.BinaryDocValuesField;
+import org.apache.lucene.document.Document;
+import org.apache.lucene.document.Field;
+import org.apache.lucene.document.FieldType;
+import org.apache.lucene.document.IntPoint;
+import org.apache.lucene.document.KnnFloatVectorField;
+import org.apache.lucene.document.NumericDocValuesField;
+import org.apache.lucene.document.SortedDocValuesField;
+import org.apache.lucene.document.SortedNumericDocValuesField;
+import org.apache.lucene.document.SortedSetDocValuesField;
+import org.apache.lucene.document.StringField;
 import org.apache.lucene.index.ExitableDirectoryReader.ExitingReaderException;
 import org.apache.lucene.search.DocIdSetIterator;
 import org.apache.lucene.search.IndexSearcher;
@@ -154,17 +164,6 @@ public class TestExitableDirectoryReader extends LuceneTestCase {
     searcher = new IndexSearcher(reader);
     searcher.search(query, 10);
     reader.close();
-
-    // Set a negative time allowed and expect the query to complete (should disable timeouts)
-    // Not checking the validity of the result, all we are bothered about in this test is the timing
-    // out.
-    directoryReader = DirectoryReader.open(directory);
-    exitableDirectoryReader = new ExitableDirectoryReader(directoryReader, disabledQueryTimeout());
-    reader = new TestReader(getOnlyLeafReader(exitableDirectoryReader));
-    searcher = new IndexSearcher(reader);
-    searcher.search(query, 10);
-    reader.close();
-
     directory.close();
   }
 
@@ -277,48 +276,40 @@ public class TestExitableDirectoryReader extends LuceneTestCase {
     searcher = new IndexSearcher(reader);
     searcher.search(query, 10);
     reader.close();
-
-    // Set a negative time allowed and expect the query to complete (should disable timeouts)
-    // Not checking the validity of the result, all we are bothered about in this test is the timing
-    // out.
-    directoryReader = DirectoryReader.open(directory);
-    exitableDirectoryReader = new ExitableDirectoryReader(directoryReader, disabledQueryTimeout());
-    reader = new TestReader(getOnlyLeafReader(exitableDirectoryReader));
-    searcher = new IndexSearcher(reader);
-    searcher.search(query, 10);
-    reader.close();
-
     directory.close();
   }
 
-  private static QueryTimeout disabledQueryTimeout() {
-    return new QueryTimeout() {
+  public void testExitableTermsMinAndMax() throws IOException {
+    Directory directory = newDirectory();
+    IndexWriter w = new IndexWriter(directory, newIndexWriterConfig(new MockAnalyzer(random())));
+    Document doc = new Document();
+    StringField fooField = new StringField("foo", "bar", Field.Store.NO);
+    doc.add(fooField);
+    w.addDocument(doc);
+    w.flush();
 
-      @Override
-      public boolean shouldExit() {
-        return false;
-      }
+    DirectoryReader directoryReader = DirectoryReader.open(w);
+    for (LeafReaderContext lfc : directoryReader.leaves()) {
+      ExitableDirectoryReader.ExitableTerms terms =
+          new ExitableDirectoryReader.ExitableTerms(
+              lfc.reader().terms("foo"), infiniteQueryTimeout()) {
+            @Override
+            public TermsEnum iterator() {
+              fail("min and max should be retrieved from block tree, no need to iterate");
+              return null;
+            }
+          };
+      assertEquals("bar", terms.getMin().utf8ToString());
+      assertEquals("bar", terms.getMax().utf8ToString());
+    }
 
-      @Override
-      public boolean isTimeoutEnabled() {
-        return false;
-      }
-    };
+    w.close();
+    directoryReader.close();
+    directory.close();
   }
 
   private static QueryTimeout infiniteQueryTimeout() {
-    return new QueryTimeout() {
-
-      @Override
-      public boolean shouldExit() {
-        return false;
-      }
-
-      @Override
-      public boolean isTimeoutEnabled() {
-        return true;
-      }
-    };
+    return () -> false;
   }
 
   private static class CountingQueryTimeout implements QueryTimeout {
@@ -330,29 +321,13 @@ public class TestExitableDirectoryReader extends LuceneTestCase {
       return false;
     }
 
-    @Override
-    public boolean isTimeoutEnabled() {
-      return true;
-    }
-
     public int getShouldExitCallCount() {
       return counter;
     }
   }
 
   private static QueryTimeout immediateQueryTimeout() {
-    return new QueryTimeout() {
-
-      @Override
-      public boolean shouldExit() {
-        return true;
-      }
-
-      @Override
-      public boolean isTimeoutEnabled() {
-        return true;
-      }
-    };
+    return () -> true;
   }
 
   @FunctionalInterface
@@ -410,9 +385,7 @@ public class TestExitableDirectoryReader extends LuceneTestCase {
 
       directoryReader = DirectoryReader.open(directory);
       exitableDirectoryReader =
-          new ExitableDirectoryReader(
-              directoryReader,
-              random().nextBoolean() ? infiniteQueryTimeout() : disabledQueryTimeout());
+          new ExitableDirectoryReader(directoryReader, infiniteQueryTimeout());
       {
         IndexReader reader = new TestReader(getOnlyLeafReader(exitableDirectoryReader));
         final LeafReader leaf = reader.leaves().get(0).reader();
@@ -447,8 +420,8 @@ public class TestExitableDirectoryReader extends LuceneTestCase {
         value[j] = random().nextFloat();
       }
       FieldType fieldType =
-          KnnVectorField.createFieldType(dimension, VectorSimilarityFunction.COSINE);
-      doc.add(new KnnVectorField("vector", value, fieldType));
+          KnnFloatVectorField.createFieldType(dimension, VectorSimilarityFunction.COSINE);
+      doc.add(new KnnFloatVectorField("vector", value, fieldType));
 
       doc.add(new StringField("id", Integer.toString(i), Field.Store.YES));
       writer.addDocument(doc);
@@ -465,18 +438,14 @@ public class TestExitableDirectoryReader extends LuceneTestCase {
 
     QueryTimeout queryTimeout;
     if (random().nextBoolean()) {
-      if (random().nextBoolean()) {
-        queryTimeout = immediateQueryTimeout();
-      } else {
-        queryTimeout = infiniteQueryTimeout();
-      }
+      queryTimeout = immediateQueryTimeout();
     } else {
-      queryTimeout = disabledQueryTimeout();
+      queryTimeout = infiniteQueryTimeout();
     }
 
     DirectoryReader directoryReader = DirectoryReader.open(directory);
-    DirectoryReader exitableDirectoryReader =
-        new ExitableDirectoryReader(directoryReader, queryTimeout);
+    DirectoryReader exitableDirectoryReader = directoryReader;
+    exitableDirectoryReader = new ExitableDirectoryReader(directoryReader, queryTimeout);
     IndexReader reader = new TestReader(getOnlyLeafReader(exitableDirectoryReader));
 
     LeafReaderContext context = reader.leaves().get(0);
@@ -486,7 +455,7 @@ public class TestExitableDirectoryReader extends LuceneTestCase {
       expectThrows(
           ExitingReaderException.class,
           () -> {
-            DocIdSetIterator iter = leaf.getVectorValues("vector");
+            DocIdSetIterator iter = leaf.getFloatVectorValues("vector");
             scanAndRetrieve(leaf, iter);
           });
 
@@ -496,7 +465,7 @@ public class TestExitableDirectoryReader extends LuceneTestCase {
               leaf.searchNearestVectors(
                   "vector", new float[dimension], 5, leaf.getLiveDocs(), Integer.MAX_VALUE));
     } else {
-      DocIdSetIterator iter = leaf.getVectorValues("vector");
+      DocIdSetIterator iter = leaf.getFloatVectorValues("vector");
       scanAndRetrieve(leaf, iter);
 
       leaf.searchNearestVectors(
@@ -519,8 +488,8 @@ public class TestExitableDirectoryReader extends LuceneTestCase {
 
       if (random().nextBoolean()
           && iter.docID() != DocIdSetIterator.NO_MORE_DOCS
-          && iter instanceof VectorValues) {
-        ((VectorValues) iter).vectorValue();
+          && iter instanceof FloatVectorValues) {
+        ((FloatVectorValues) iter).vectorValue();
       }
     }
   }
