@@ -16,8 +16,6 @@
  */
 package org.apache.lucene.index;
 
-
-
 import java.io.IOException;
 import java.util.BitSet;
 import java.util.Collection;
@@ -25,36 +23,34 @@ import java.util.HashMap;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
-
-import org.apache.lucene.analysis.MockAnalyzer;
 import org.apache.lucene.document.Document;
 import org.apache.lucene.document.Field;
 import org.apache.lucene.store.Directory;
+import org.apache.lucene.tests.analysis.MockAnalyzer;
+import org.apache.lucene.tests.util.LuceneTestCase;
 import org.apache.lucene.util.Bits;
-import org.apache.lucene.util.LuceneTestCase;
 
 /**
  * Test class to illustrate using IndexDeletionPolicy to provide multi-level rollback capability.
  * This test case creates an index of records 1 to 100, introducing a commit point every 10 records.
- * 
- * A "keep all" deletion policy is used to ensure we keep all commit points for testing purposes
+ *
+ * <p>A "keep all" deletion policy is used to ensure we keep all commit points for testing purposes
  */
-
 public class TestTransactionRollback extends LuceneTestCase {
 
   private static final String FIELD_RECORD_ID = "record_id";
   private Directory dir;
 
-  //Rolls back index to a chosen ID
+  // Rolls back index to a chosen ID
   private void rollBackLast(int id) throws Exception {
 
     // System.out.println("Attempting to rollback to "+id);
-    String ids="-"+id;
-    IndexCommit last=null;
+    String ids = "-" + id;
+    IndexCommit last = null;
     Collection<IndexCommit> commits = DirectoryReader.listCommits(dir);
-    for (Iterator<IndexCommit> iterator = commits.iterator(); iterator.hasNext();) {
-      IndexCommit commit =  iterator.next();
-      Map<String,String> ud=commit.getUserData();
+    for (Iterator<IndexCommit> iterator = commits.iterator(); iterator.hasNext(); ) {
+      IndexCommit commit = iterator.next();
+      Map<String, String> ud = commit.getUserData();
       if (ud.size() > 0) {
         if (ud.get("index").endsWith(ids)) {
           last = commit;
@@ -62,15 +58,18 @@ public class TestTransactionRollback extends LuceneTestCase {
       }
     }
 
-    if (last==null) {
-      throw new RuntimeException("Couldn't find commit point "+id);
+    if (last == null) {
+      throw new RuntimeException("Couldn't find commit point " + id);
     }
 
-    IndexWriter w = new IndexWriter(dir, newIndexWriterConfig(new MockAnalyzer(random()))
-                                           .setIndexDeletionPolicy(new RollbackDeletionPolicy(id))
-                                           .setIndexCommit(last));
-    Map<String,String> data = new HashMap<>();
-    data.put("index", "Rolled back to 1-"+id);
+    IndexWriter w =
+        new IndexWriter(
+            dir,
+            newIndexWriterConfig(new MockAnalyzer(random()))
+                .setIndexDeletionPolicy(new RollbackDeletionPolicy(id))
+                .setIndexCommit(last));
+    Map<String, String> data = new HashMap<>();
+    data.put("index", "Rolled back to 1-" + id);
     w.setLiveCommitData(data.entrySet());
     w.close();
   }
@@ -78,12 +77,12 @@ public class TestTransactionRollback extends LuceneTestCase {
   public void testRepeatedRollBacks() throws Exception {
 
     int expectedLastRecordId = 100;
-    while (expectedLastRecordId>10) {
+    while (expectedLastRecordId > 10) {
       expectedLastRecordId -= 10;
       rollBackLast(expectedLastRecordId);
-      
+
       BitSet expecteds = new BitSet(100);
-      expecteds.set(1, (expectedLastRecordId+1), true);
+      expecteds.set(1, (expectedLastRecordId + 1), true);
       checkExpecteds(expecteds);
     }
   }
@@ -91,21 +90,22 @@ public class TestTransactionRollback extends LuceneTestCase {
   private void checkExpecteds(BitSet expecteds) throws Exception {
     IndexReader r = DirectoryReader.open(dir);
 
-    //Perhaps not the most efficient approach but meets our
-    //needs here.
+    // Perhaps not the most efficient approach but meets our
+    // needs here.
     final Bits liveDocs = MultiBits.getLiveDocs(r);
+    StoredFields storedFields = r.storedFields();
     for (int i = 0; i < r.maxDoc(); i++) {
       if (liveDocs == null || liveDocs.get(i)) {
-        String sval=r.document(i).get(FIELD_RECORD_ID);
-        if(sval!=null) {
-          int val=Integer.parseInt(sval);
-          assertTrue("Did not expect document #"+val, expecteds.get(val));
-          expecteds.set(val,false);
+        String sval = storedFields.document(i).get(FIELD_RECORD_ID);
+        if (sval != null) {
+          int val = Integer.parseInt(sval);
+          assertTrue("Did not expect document #" + val, expecteds.get(val));
+          expecteds.set(val, false);
         }
       }
     }
     r.close();
-    assertEquals("Should have 0 docs remaining ", 0 ,expecteds.cardinality());
+    assertEquals("Should have 0 docs remaining ", 0, expecteds.cardinality());
   }
 
   /*
@@ -129,19 +129,20 @@ public class TestTransactionRollback extends LuceneTestCase {
     super.setUp();
     dir = newDirectory();
 
-    //Build index, of records 1 to 100, committing after each batch of 10
-    IndexDeletionPolicy sdp=new KeepAllDeletionPolicy();
-    IndexWriter w=new IndexWriter(dir, newIndexWriterConfig(new MockAnalyzer(random()))
-                                          .setIndexDeletionPolicy(sdp));
+    // Build index, of records 1 to 100, committing after each batch of 10
+    IndexDeletionPolicy sdp = new KeepAllDeletionPolicy();
+    IndexWriter w =
+        new IndexWriter(
+            dir, newIndexWriterConfig(new MockAnalyzer(random())).setIndexDeletionPolicy(sdp));
 
-    for(int currentRecordId=1;currentRecordId<=100;currentRecordId++) {
-      Document doc=new Document();
-      doc.add(newTextField(FIELD_RECORD_ID, ""+currentRecordId, Field.Store.YES));
+    for (int currentRecordId = 1; currentRecordId <= 100; currentRecordId++) {
+      Document doc = new Document();
+      doc.add(newTextField(FIELD_RECORD_ID, "" + currentRecordId, Field.Store.YES));
       w.addDocument(doc);
 
-      if (currentRecordId%10 == 0) {
-        Map<String,String> data = new HashMap<>();
-        data.put("index", "records 1-"+currentRecordId);
+      if (currentRecordId % 10 == 0) {
+        Map<String, String> data = new HashMap<>();
+        data.put("index", "records 1-" + currentRecordId);
         w.setLiveCommitData(data.entrySet());
         w.commit();
       }
@@ -149,7 +150,7 @@ public class TestTransactionRollback extends LuceneTestCase {
 
     w.close();
   }
-  
+
   @Override
   public void tearDown() throws Exception {
     dir.close();
@@ -165,21 +166,20 @@ public class TestTransactionRollback extends LuceneTestCase {
     }
 
     @Override
-    public void onCommit(List<? extends IndexCommit> commits) throws IOException {
-    }
+    public void onCommit(List<? extends IndexCommit> commits) throws IOException {}
 
     @Override
     public void onInit(List<? extends IndexCommit> commits) throws IOException {
       for (final IndexCommit commit : commits) {
-        Map<String,String> userData=commit.getUserData();
+        Map<String, String> userData = commit.getUserData();
         if (userData.size() > 0) {
           // Label for a commit point is "Records 1-30"
           // This code reads the last id ("30" in this example) and deletes it
           // if it is after the desired rollback point
           String x = userData.get("index");
-          String lastVal = x.substring(x.lastIndexOf("-")+1);
+          String lastVal = x.substring(x.lastIndexOf("-") + 1);
           int last = Integer.parseInt(lastVal);
-          if (last>rollbackPoint) {
+          if (last > rollbackPoint) {
             /*
             System.out.print("\tRolling back commit point:" +
                              " UserData="+commit.getUserData() +")  ("+(commits.size()-1)+" commit points left) files=");
@@ -204,17 +204,20 @@ public class TestTransactionRollback extends LuceneTestCase {
 
     @Override
     public void onInit(List<? extends IndexCommit> commits) throws IOException {
-      commits.get(commits.size()-1).delete();
+      commits.get(commits.size() - 1).delete();
     }
   }
 
   public void testRollbackDeletionPolicy() throws Exception {
 
-    for(int i=0;i<2;i++) {
+    for (int i = 0; i < 2; i++) {
       // Unless you specify a prior commit point, rollback
       // should not work:
-      new IndexWriter(dir, newIndexWriterConfig(new MockAnalyzer(random()))
-          .setIndexDeletionPolicy(new DeleteLastCommitPolicy())).close();
+      new IndexWriter(
+              dir,
+              newIndexWriterConfig(new MockAnalyzer(random()))
+                  .setIndexDeletionPolicy(new DeleteLastCommitPolicy()))
+          .close();
       IndexReader r = DirectoryReader.open(dir);
       assertEquals(100, r.numDocs());
       r.close();
@@ -225,6 +228,7 @@ public class TestTransactionRollback extends LuceneTestCase {
   static class KeepAllDeletionPolicy extends IndexDeletionPolicy {
     @Override
     public void onCommit(List<? extends IndexCommit> commits) throws IOException {}
+
     @Override
     public void onInit(List<? extends IndexCommit> commits) throws IOException {}
   }
