@@ -116,11 +116,12 @@ public class TestElevationComparator extends LuceneTestCase {
     }
 
     /*
+     StoredFields storedFields = searcher.storedFields();
      for (int i = 0; i < nDocsReturned; i++) {
       ScoreDoc scoreDoc = topDocs.scoreDocs[i];
       ids[i] = scoreDoc.doc;
       scores[i] = scoreDoc.score;
-      documents[i] = searcher.doc(ids[i]);
+      documents[i] = storedFields.document(ids[i]);
       System.out.println("ids[i] = " + ids[i]);
       System.out.println("documents[i] = " + documents[i]);
       System.out.println("scores[i] = " + scores[i]);
