@@ -34,6 +34,7 @@ import org.apache.lucene.document.StringField;
 import org.apache.lucene.index.DirectoryReader;
 import org.apache.lucene.index.IndexReader;
 import org.apache.lucene.index.MultiReader;
+import org.apache.lucene.index.StoredFields;
 import org.apache.lucene.index.Term;
 import org.apache.lucene.search.BooleanClause.Occur;
 import org.apache.lucene.search.similarities.ClassicSimilarity;
@@ -112,8 +113,9 @@ public class TestFuzzyQuery extends LuceneTestCase {
     hits = searcher.search(query, 1000).scoreDocs;
     assertEquals("3 documents should match", 3, hits.length);
     List<String> order = Arrays.asList("bbbbb", "abbbb", "aabbb");
+    StoredFields storedFields = searcher.storedFields();
     for (int i = 0; i < hits.length; i++) {
-      final String term = searcher.doc(hits[i].doc).get("field");
+      final String term = storedFields.document(hits[i].doc).get("field");
       // System.out.println(hits[i].score);
       assertEquals(order.get(i), term);
     }
@@ -125,7 +127,7 @@ public class TestFuzzyQuery extends LuceneTestCase {
     assertEquals("only 2 documents should match", 2, hits.length);
     order = Arrays.asList("bbbbb", "abbbb");
     for (int i = 0; i < hits.length; i++) {
-      final String term = searcher.doc(hits[i].doc).get("field");
+      final String term = storedFields.document(hits[i].doc).get("field");
       // System.out.println(hits[i].score);
       assertEquals(order.get(i), term);
     }
@@ -146,43 +148,43 @@ public class TestFuzzyQuery extends LuceneTestCase {
     query = new FuzzyQuery(new Term("field", "aaaaa"), FuzzyQuery.defaultMaxEdits, 0);
     hits = searcher.search(query, 1000).scoreDocs;
     assertEquals(3, hits.length);
-    assertEquals(searcher.doc(hits[0].doc).get("field"), ("aaaaa"));
+    assertEquals(storedFields.document(hits[0].doc).get("field"), ("aaaaa"));
     // default allows for up to two edits:
-    assertEquals(searcher.doc(hits[1].doc).get("field"), ("aaaab"));
-    assertEquals(searcher.doc(hits[2].doc).get("field"), ("aaabb"));
+    assertEquals(storedFields.document(hits[1].doc).get("field"), ("aaaab"));
+    assertEquals(storedFields.document(hits[2].doc).get("field"), ("aaabb"));
 
     // query similar to a word in the index:
     query = new FuzzyQuery(new Term("field", "aaaac"), FuzzyQuery.defaultMaxEdits, 0);
     hits = searcher.search(query, 1000).scoreDocs;
     assertEquals(3, hits.length);
-    assertEquals(searcher.doc(hits[0].doc).get("field"), ("aaaaa"));
-    assertEquals(searcher.doc(hits[1].doc).get("field"), ("aaaab"));
-    assertEquals(searcher.doc(hits[2].doc).get("field"), ("aaabb"));
+    assertEquals(storedFields.document(hits[0].doc).get("field"), ("aaaaa"));
+    assertEquals(storedFields.document(hits[1].doc).get("field"), ("aaaab"));
+    assertEquals(storedFields.document(hits[2].doc).get("field"), ("aaabb"));
 
     // now with prefix
     query = new FuzzyQuery(new Term("field", "aaaac"), FuzzyQuery.defaultMaxEdits, 1);
     hits = searcher.search(query, 1000).scoreDocs;
     assertEquals(3, hits.length);
-    assertEquals(searcher.doc(hits[0].doc).get("field"), ("aaaaa"));
-    assertEquals(searcher.doc(hits[1].doc).get("field"), ("aaaab"));
-    assertEquals(searcher.doc(hits[2].doc).get("field"), ("aaabb"));
+    assertEquals(storedFields.document(hits[0].doc).get("field"), ("aaaaa"));
+    assertEquals(storedFields.document(hits[1].doc).get("field"), ("aaaab"));
+    assertEquals(storedFields.document(hits[2].doc).get("field"), ("aaabb"));
     query = new FuzzyQuery(new Term("field", "aaaac"), FuzzyQuery.defaultMaxEdits, 2);
     hits = searcher.search(query, 1000).scoreDocs;
     assertEquals(3, hits.length);
-    assertEquals(searcher.doc(hits[0].doc).get("field"), ("aaaaa"));
-    assertEquals(searcher.doc(hits[1].doc).get("field"), ("aaaab"));
-    assertEquals(searcher.doc(hits[2].doc).get("field"), ("aaabb"));
+    assertEquals(storedFields.document(hits[0].doc).get("field"), ("aaaaa"));
+    assertEquals(storedFields.document(hits[1].doc).get("field"), ("aaaab"));
+    assertEquals(storedFields.document(hits[2].doc).get("field"), ("aaabb"));
     query = new FuzzyQuery(new Term("field", "aaaac"), FuzzyQuery.defaultMaxEdits, 3);
     hits = searcher.search(query, 1000).scoreDocs;
     assertEquals(3, hits.length);
-    assertEquals(searcher.doc(hits[0].doc).get("field"), ("aaaaa"));
-    assertEquals(searcher.doc(hits[1].doc).get("field"), ("aaaab"));
-    assertEquals(searcher.doc(hits[2].doc).get("field"), ("aaabb"));
+    assertEquals(storedFields.document(hits[0].doc).get("field"), ("aaaaa"));
+    assertEquals(storedFields.document(hits[1].doc).get("field"), ("aaaab"));
+    assertEquals(storedFields.document(hits[2].doc).get("field"), ("aaabb"));
     query = new FuzzyQuery(new Term("field", "aaaac"), FuzzyQuery.defaultMaxEdits, 4);
     hits = searcher.search(query, 1000).scoreDocs;
     assertEquals(2, hits.length);
-    assertEquals(searcher.doc(hits[0].doc).get("field"), ("aaaaa"));
-    assertEquals(searcher.doc(hits[1].doc).get("field"), ("aaaab"));
+    assertEquals(storedFields.document(hits[0].doc).get("field"), ("aaaaa"));
+    assertEquals(storedFields.document(hits[1].doc).get("field"), ("aaaab"));
     query = new FuzzyQuery(new Term("field", "aaaac"), FuzzyQuery.defaultMaxEdits, 5);
     hits = searcher.search(query, 1000).scoreDocs;
     assertEquals(0, hits.length);
@@ -190,25 +192,25 @@ public class TestFuzzyQuery extends LuceneTestCase {
     query = new FuzzyQuery(new Term("field", "ddddX"), FuzzyQuery.defaultMaxEdits, 0);
     hits = searcher.search(query, 1000).scoreDocs;
     assertEquals(1, hits.length);
-    assertEquals(searcher.doc(hits[0].doc).get("field"), ("ddddd"));
+    assertEquals(storedFields.document(hits[0].doc).get("field"), ("ddddd"));
 
     // now with prefix
     query = new FuzzyQuery(new Term("field", "ddddX"), FuzzyQuery.defaultMaxEdits, 1);
     hits = searcher.search(query, 1000).scoreDocs;
     assertEquals(1, hits.length);
-    assertEquals(searcher.doc(hits[0].doc).get("field"), ("ddddd"));
+    assertEquals(storedFields.document(hits[0].doc).get("field"), ("ddddd"));
     query = new FuzzyQuery(new Term("field", "ddddX"), FuzzyQuery.defaultMaxEdits, 2);
     hits = searcher.search(query, 1000).scoreDocs;
     assertEquals(1, hits.length);
-    assertEquals(searcher.doc(hits[0].doc).get("field"), ("ddddd"));
+    assertEquals(storedFields.document(hits[0].doc).get("field"), ("ddddd"));
     query = new FuzzyQuery(new Term("field", "ddddX"), FuzzyQuery.defaultMaxEdits, 3);
     hits = searcher.search(query, 1000).scoreDocs;
     assertEquals(1, hits.length);
-    assertEquals(searcher.doc(hits[0].doc).get("field"), ("ddddd"));
+    assertEquals(storedFields.document(hits[0].doc).get("field"), ("ddddd"));
     query = new FuzzyQuery(new Term("field", "ddddX"), FuzzyQuery.defaultMaxEdits, 4);
     hits = searcher.search(query, 1000).scoreDocs;
     assertEquals(1, hits.length);
-    assertEquals(searcher.doc(hits[0].doc).get("field"), ("ddddd"));
+    assertEquals(storedFields.document(hits[0].doc).get("field"), ("ddddd"));
     query = new FuzzyQuery(new Term("field", "ddddX"), FuzzyQuery.defaultMaxEdits, 5);
     hits = searcher.search(query, 1000).scoreDocs;
     assertEquals(0, hits.length);
@@ -319,15 +321,16 @@ public class TestFuzzyQuery extends LuceneTestCase {
         new ClassicSimilarity()); // avoid randomisation of similarity algo by test framework
     writer.close();
     String[] searchTerms = {"smith", "smythe", "smdssasd"};
+    StoredFields storedFields = reader.storedFields();
     for (String searchTerm : searchTerms) {
       FuzzyQuery query = new FuzzyQuery(new Term("field", searchTerm), 2, 1);
       ScoreDoc[] hits = searcher.search(query, 1000).scoreDocs;
-      Document bestDoc = searcher.doc(hits[0].doc);
+      Document bestDoc = storedFields.document(hits[0].doc);
       assertTrue(hits.length > 0);
       String topMatch = bestDoc.get("field");
       assertEquals(searchTerm, topMatch);
       if (hits.length > 1) {
-        Document worstDoc = searcher.doc(hits[hits.length - 1].doc);
+        Document worstDoc = storedFields.document(hits[hits.length - 1].doc);
         String worstMatch = worstDoc.get("field");
         assertNotSame(searchTerm, worstMatch);
       }
@@ -371,15 +374,15 @@ public class TestFuzzyQuery extends LuceneTestCase {
 
     // Matches on the rare surname should be worth more than matches on the common forename
     assertEquals(7, hits.length);
-    Document bestDoc = searcher.doc(hits[0].doc);
+    Document bestDoc = searcher.storedFields().document(hits[0].doc);
     String topMatch = bestDoc.get("field");
     assertTrue(topMatch.contains(rareSearchTerm));
 
-    Document runnerUpDoc = searcher.doc(hits[1].doc);
+    Document runnerUpDoc = searcher.storedFields().document(hits[1].doc);
     String runnerUpMatch = runnerUpDoc.get("field");
     assertTrue(runnerUpMatch.contains("cuttin"));
 
-    Document worstDoc = searcher.doc(hits[hits.length - 1].doc);
+    Document worstDoc = searcher.storedFields().document(hits[hits.length - 1].doc);
     String worstMatch = worstDoc.get("field");
     assertTrue(worstMatch.contains("micheal")); // misspelling of common name
 
@@ -451,9 +454,9 @@ public class TestFuzzyQuery extends LuceneTestCase {
     ScoreDoc[] hits = searcher.search(query, 1000).scoreDocs;
     assertEquals(3, hits.length);
     // normally, 'Lucenne' would be the first result as IDF will skew the score.
-    assertEquals("Lucene", reader.document(hits[0].doc).get("field"));
-    assertEquals("Lucene", reader.document(hits[1].doc).get("field"));
-    assertEquals("Lucenne", reader.document(hits[2].doc).get("field"));
+    assertEquals("Lucene", reader.storedFields().document(hits[0].doc).get("field"));
+    assertEquals("Lucene", reader.storedFields().document(hits[1].doc).get("field"));
+    assertEquals("Lucenne", reader.storedFields().document(hits[2].doc).get("field"));
     reader.close();
     directory.close();
   }
@@ -489,7 +492,7 @@ public class TestFuzzyQuery extends LuceneTestCase {
     IndexSearcher searcher = newSearcher(r);
     ScoreDoc[] hits = searcher.search(q, 10).scoreDocs;
     assertEquals(1, hits.length);
-    assertEquals("Giga byte", searcher.doc(hits[0].doc).get("field"));
+    assertEquals("Giga byte", searcher.storedFields().document(hits[0].doc).get("field"));
     r.close();
     w.close();
     index.close();
@@ -508,12 +511,12 @@ public class TestFuzzyQuery extends LuceneTestCase {
     FuzzyQuery q = new FuzzyQuery(new Term("field", "fouba"), 2);
     ScoreDoc[] hits = searcher.search(q, 10).scoreDocs;
     assertEquals(1, hits.length);
-    assertEquals("foobar", searcher.doc(hits[0].doc).get("field"));
+    assertEquals("foobar", searcher.storedFields().document(hits[0].doc).get("field"));
 
     q = new FuzzyQuery(new Term("field", "foubara"), 2);
     hits = searcher.search(q, 10).scoreDocs;
     assertEquals(1, hits.length);
-    assertEquals("foobar", searcher.doc(hits[0].doc).get("field"));
+    assertEquals("foobar", searcher.storedFields().document(hits[0].doc).get("field"));
 
     expectThrows(
         IllegalArgumentException.class,
@@ -676,8 +679,9 @@ public class TestFuzzyQuery extends LuceneTestCase {
             new FuzzyQuery(new Term("field", queryTerm), ed, prefixLength, queueSize, true);
         TopDocs hits = s.search(query, terms.size());
         Set<String> actual = new HashSet<>();
+        StoredFields storedFields = s.storedFields();
         for (ScoreDoc hit : hits.scoreDocs) {
-          Document doc = s.doc(hit.doc);
+          Document doc = storedFields.document(hit.doc);
           actual.add(doc.get("field"));
           // System.out.println("   actual: " + doc.get("field") + " score=" + hit.score);
         }
