@@ -73,6 +73,18 @@ public class TestSynonymQuery extends LuceneTestCase {
             .addTerm(new Term("field", "c"), 0.2f)
             .addTerm(new Term("field", "d"))
             .build());
+
+    QueryUtils.checkUnequal(
+        new SynonymQuery.Builder("field").addTerm(new Term("field", "a"), 0.4f).build(),
+        new SynonymQuery.Builder("field").addTerm(new Term("field", "b"), 0.4f).build());
+
+    QueryUtils.checkUnequal(
+        new SynonymQuery.Builder("field").addTerm(new Term("field", "a"), 0.2f).build(),
+        new SynonymQuery.Builder("field").addTerm(new Term("field", "a"), 0.4f).build());
+
+    QueryUtils.checkUnequal(
+        new SynonymQuery.Builder("field1").addTerm(new Term("field1", "b"), 0.4f).build(),
+        new SynonymQuery.Builder("field2").addTerm(new Term("field2", "b"), 0.4f).build());
   }
 
   public void testBogusParams() {
@@ -127,6 +139,12 @@ public class TestSynonymQuery extends LuceneTestCase {
         () -> {
           new SynonymQuery.Builder("field1").addTerm(new Term("field1", "a"), -0f);
         });
+
+    expectThrows(
+        NullPointerException.class,
+        () -> new SynonymQuery.Builder(null).addTerm(new Term("field1", "a"), -0f));
+
+    expectThrows(NullPointerException.class, () -> new SynonymQuery.Builder(null).build());
   }
 
   public void testToString() {
