@@ -18,7 +18,9 @@ package org.apache.lucene.search;
 
 import java.io.IOException;
 import java.util.Arrays;
-
+import java.util.List;
+import java.util.Objects;
+import java.util.stream.Collectors;
 import org.apache.lucene.document.Document;
 import org.apache.lucene.document.Field.Store;
 import org.apache.lucene.document.StringField;
@@ -29,8 +31,11 @@ import org.apache.lucene.index.LeafReaderContext;
 import org.apache.lucene.index.Term;
 import org.apache.lucene.search.BooleanClause.Occur;
 import org.apache.lucene.store.Directory;
-import org.apache.lucene.util.LuceneTestCase;
-import org.apache.lucene.util.TestUtil;
+import org.apache.lucene.tests.search.AssertingQuery;
+import org.apache.lucene.tests.search.BlockScoreQueryWrapper;
+import org.apache.lucene.tests.search.CheckHits;
+import org.apache.lucene.tests.util.LuceneTestCase;
+import org.apache.lucene.tests.util.TestUtil;
 
 public class TestWANDScorer extends LuceneTestCase {
 
@@ -43,15 +48,33 @@ public class TestWANDScorer extends LuceneTestCase {
     doTestScalingFactor(Math.nextUp(Float.MIN_VALUE));
     doTestScalingFactor(Float.MAX_VALUE);
     doTestScalingFactor(Math.nextDown(Float.MAX_VALUE));
-    assertEquals(WANDScorer.scalingFactor(Float.MIN_VALUE) - 1, WANDScorer.scalingFactor(0));
-    assertEquals(WANDScorer.scalingFactor(Float.MAX_VALUE) + 1, WANDScorer.scalingFactor(Float.POSITIVE_INFINITY));
+    assertEquals(WANDScorer.scalingFactor(Float.MIN_VALUE) + 1, WANDScorer.scalingFactor(0));
+    assertEquals(
+        WANDScorer.scalingFactor(Float.MAX_VALUE) - 1,
+        WANDScorer.scalingFactor(Float.POSITIVE_INFINITY));
+
+    // Greater scores produce lower scaling factors
+    assertTrue(WANDScorer.scalingFactor(1f) > WANDScorer.scalingFactor(10f));
+    assertTrue(
+        WANDScorer.scalingFactor(Float.MAX_VALUE)
+            > WANDScorer.scalingFactor(Float.POSITIVE_INFINITY));
+    assertTrue(WANDScorer.scalingFactor(0f) > WANDScorer.scalingFactor(Float.MIN_VALUE));
   }
 
   private void doTestScalingFactor(float f) {
     int scalingFactor = WANDScorer.scalingFactor(f);
     float scaled = Math.scalb(f, scalingFactor);
-    assertTrue(""+scaled, scaled > 1 << 15);
-    assertTrue(""+scaled, scaled <= 1 << 16);
+    assertTrue("" + scaled, scaled >= 1 << (WANDScorer.FLOAT_MANTISSA_BITS - 1));
+    assertTrue("" + scaled, scaled < 1 << WANDScorer.FLOAT_MANTISSA_BITS);
+  }
+
+  public void testScaleMaxScore() {
+    assertEquals(
+        1 << (WANDScorer.FLOAT_MANTISSA_BITS - 1),
+        WANDScorer.scaleMaxScore(32f, WANDScorer.scalingFactor(32f)));
+    assertEquals(1, WANDScorer.scaleMaxScore(32f, WANDScorer.scalingFactor(Math.scalb(1f, 60))));
+    assertEquals(
+        1, WANDScorer.scaleMaxScore(32f, WANDScorer.scalingFactor(Float.POSITIVE_INFINITY)));
   }
 
   private Query maybeWrap(Query query) {
@@ -64,15 +87,17 @@ public class TestWANDScorer extends LuceneTestCase {
 
   public void testBasics() throws Exception {
     Directory dir = newDirectory();
-    IndexWriter w = new IndexWriter(dir, newIndexWriterConfig().setMergePolicy(newLogMergePolicy()));
-    for (String[] values : Arrays.asList(
-        new String[]{ "A", "B" },       // 0
-        new String[]{ "A" },            // 1
-        new String[]{ },                // 2
-        new String[]{ "A", "B", "C" },  // 3
-        new String[]{ "B" },            // 4
-        new String[]{ "B", "C" }        // 5
-        )) {
+    IndexWriter w =
+        new IndexWriter(dir, newIndexWriterConfig().setMergePolicy(newLogMergePolicy()));
+    for (String[] values :
+        Arrays.asList(
+            new String[] {"A", "B"}, // 0
+            new String[] {"A"}, // 1
+            new String[] {}, // 2
+            new String[] {"A", "B", "C"}, // 3
+            new String[] {"B"}, // 4
+            new String[] {"B", "C"} // 5
+            )) {
       Document doc = new Document();
       for (String value : values) {
         doc.add(new StringField("foo", value, Store.NO));
@@ -85,15 +110,22 @@ public class TestWANDScorer extends LuceneTestCase {
     IndexReader reader = DirectoryReader.open(dir);
     IndexSearcher searcher = newSearcher(reader);
 
-    Query query = new BooleanQuery.Builder()
-        .add(new BoostQuery(new ConstantScoreQuery(new TermQuery(new Term("foo", "A"))), 2), Occur.SHOULD)
-        .add(new ConstantScoreQuery(new TermQuery(new Term("foo", "B"))), Occur.SHOULD)
-        .add(new BoostQuery(new ConstantScoreQuery(new TermQuery(new Term("foo", "C"))), 3), Occur.SHOULD)
-        .build();
+    Query query =
+        new WANDScorerQuery(
+            new BooleanQuery.Builder()
+                .add(
+                    new BoostQuery(new ConstantScoreQuery(new TermQuery(new Term("foo", "A"))), 2),
+                    Occur.SHOULD)
+                .add(new ConstantScoreQuery(new TermQuery(new Term("foo", "B"))), Occur.SHOULD)
+                .add(
+                    new BoostQuery(new ConstantScoreQuery(new TermQuery(new Term("foo", "C"))), 3),
+                    Occur.SHOULD)
+                .build());
 
-    Scorer scorer = searcher
-        .createWeight(searcher.rewrite(query), ScoreMode.TOP_SCORES, 1)
-        .scorer(searcher.getIndexReader().leaves().get(0));
+    Scorer scorer =
+        searcher
+            .createWeight(searcher.rewrite(query), ScoreMode.TOP_SCORES, 1)
+            .scorer(searcher.getIndexReader().leaves().get(0));
 
     assertEquals(0, scorer.iterator().nextDoc());
     assertEquals(2 + 1, scorer.score(), 0);
@@ -112,9 +144,10 @@ public class TestWANDScorer extends LuceneTestCase {
 
     assertEquals(DocIdSetIterator.NO_MORE_DOCS, scorer.iterator().nextDoc());
 
-    scorer = searcher
-        .createWeight(searcher.rewrite(query), ScoreMode.TOP_SCORES, 1)
-        .scorer(searcher.getIndexReader().leaves().get(0));
+    scorer =
+        searcher
+            .createWeight(searcher.rewrite(query), ScoreMode.TOP_SCORES, 1)
+            .scorer(searcher.getIndexReader().leaves().get(0));
     scorer.setMinCompetitiveScore(4);
 
     assertEquals(3, scorer.iterator().nextDoc());
@@ -125,9 +158,10 @@ public class TestWANDScorer extends LuceneTestCase {
 
     assertEquals(DocIdSetIterator.NO_MORE_DOCS, scorer.iterator().nextDoc());
 
-    scorer = searcher
-        .createWeight(searcher.rewrite(query), ScoreMode.TOP_SCORES, 1)
-        .scorer(searcher.getIndexReader().leaves().get(0));
+    scorer =
+        searcher
+            .createWeight(searcher.rewrite(query), ScoreMode.TOP_SCORES, 1)
+            .scorer(searcher.getIndexReader().leaves().get(0));
 
     assertEquals(0, scorer.iterator().nextDoc());
     assertEquals(2 + 1, scorer.score(), 0);
@@ -137,18 +171,27 @@ public class TestWANDScorer extends LuceneTestCase {
     assertEquals(DocIdSetIterator.NO_MORE_DOCS, scorer.iterator().nextDoc());
 
     //  test a filtered disjunction
-    query = new BooleanQuery.Builder()
-        .add(
-            new BooleanQuery.Builder()
-            .add(new BoostQuery(new ConstantScoreQuery(new TermQuery(new Term("foo", "A"))), 2), Occur.SHOULD)
-            .add(new ConstantScoreQuery(new TermQuery(new Term("foo", "B"))), Occur.SHOULD)
-            .build(), Occur.MUST)
-        .add(new TermQuery(new Term("foo", "C")), Occur.FILTER)
-        .build();
+    query =
+        new BooleanQuery.Builder()
+            .add(
+                new WANDScorerQuery(
+                    new BooleanQuery.Builder()
+                        .add(
+                            new BoostQuery(
+                                new ConstantScoreQuery(new TermQuery(new Term("foo", "A"))), 2),
+                            Occur.SHOULD)
+                        .add(
+                            new ConstantScoreQuery(new TermQuery(new Term("foo", "B"))),
+                            Occur.SHOULD)
+                        .build()),
+                Occur.MUST)
+            .add(new TermQuery(new Term("foo", "C")), Occur.FILTER)
+            .build();
 
-    scorer = searcher
-        .createWeight(searcher.rewrite(query), ScoreMode.TOP_SCORES, 1)
-        .scorer(searcher.getIndexReader().leaves().get(0));
+    scorer =
+        searcher
+            .createWeight(searcher.rewrite(query), ScoreMode.TOP_SCORES, 1)
+            .scorer(searcher.getIndexReader().leaves().get(0));
 
     assertEquals(3, scorer.iterator().nextDoc());
     assertEquals(2 + 1, scorer.score(), 0);
@@ -158,9 +201,10 @@ public class TestWANDScorer extends LuceneTestCase {
 
     assertEquals(DocIdSetIterator.NO_MORE_DOCS, scorer.iterator().nextDoc());
 
-    scorer = searcher
-        .createWeight(searcher.rewrite(query), ScoreMode.TOP_SCORES, 1)
-        .scorer(searcher.getIndexReader().leaves().get(0));
+    scorer =
+        searcher
+            .createWeight(searcher.rewrite(query), ScoreMode.TOP_SCORES, 1)
+            .scorer(searcher.getIndexReader().leaves().get(0));
 
     scorer.setMinCompetitiveScore(2);
 
@@ -170,15 +214,27 @@ public class TestWANDScorer extends LuceneTestCase {
     assertEquals(DocIdSetIterator.NO_MORE_DOCS, scorer.iterator().nextDoc());
 
     // Now test a filtered disjunction with a MUST_NOT
-    query = new BooleanQuery.Builder()
-        .add(new BoostQuery(new ConstantScoreQuery(new TermQuery(new Term("foo", "A"))), 2), Occur.SHOULD)
-        .add(new ConstantScoreQuery(new TermQuery(new Term("foo", "B"))), Occur.SHOULD)
-        .add(new TermQuery(new Term("foo", "C")), Occur.MUST_NOT)
-        .build();
+    query =
+        new BooleanQuery.Builder()
+            .add(
+                new WANDScorerQuery(
+                    new BooleanQuery.Builder()
+                        .add(
+                            new BoostQuery(
+                                new ConstantScoreQuery(new TermQuery(new Term("foo", "A"))), 2),
+                            Occur.SHOULD)
+                        .add(
+                            new ConstantScoreQuery(new TermQuery(new Term("foo", "B"))),
+                            Occur.SHOULD)
+                        .build()),
+                Occur.MUST)
+            .add(new TermQuery(new Term("foo", "C")), Occur.MUST_NOT)
+            .build();
 
-    scorer = searcher
-        .createWeight(searcher.rewrite(query), ScoreMode.TOP_SCORES, 1)
-        .scorer(searcher.getIndexReader().leaves().get(0));
+    scorer =
+        searcher
+            .createWeight(searcher.rewrite(query), ScoreMode.TOP_SCORES, 1)
+            .scorer(searcher.getIndexReader().leaves().get(0));
 
     assertEquals(0, scorer.iterator().nextDoc());
     assertEquals(2 + 1, scorer.score(), 0);
@@ -191,9 +247,10 @@ public class TestWANDScorer extends LuceneTestCase {
 
     assertEquals(DocIdSetIterator.NO_MORE_DOCS, scorer.iterator().nextDoc());
 
-    scorer = searcher
-        .createWeight(searcher.rewrite(query), ScoreMode.TOP_SCORES, 1)
-        .scorer(searcher.getIndexReader().leaves().get(0));
+    scorer =
+        searcher
+            .createWeight(searcher.rewrite(query), ScoreMode.TOP_SCORES, 1)
+            .scorer(searcher.getIndexReader().leaves().get(0));
 
     scorer.setMinCompetitiveScore(3);
 
@@ -233,16 +290,19 @@ public class TestWANDScorer extends LuceneTestCase {
         IndexSearcher searcher = newSearcher(reader);
 
         Query query =
-            new BooleanQuery.Builder()
-                .add(
-                    new BoostQuery(new ConstantScoreQuery(new TermQuery(new Term("foo", "A"))), 2),
-                    Occur.SHOULD)
-                .add(new ConstantScoreQuery(new TermQuery(new Term("foo", "B"))), Occur.SHOULD)
-                .add(
-                    new BoostQuery(new ConstantScoreQuery(new TermQuery(new Term("foo", "C"))), 3),
-                    Occur.SHOULD)
-                .setMinimumNumberShouldMatch(2)
-                .build();
+            new WANDScorerQuery(
+                new BooleanQuery.Builder()
+                    .add(
+                        new BoostQuery(
+                            new ConstantScoreQuery(new TermQuery(new Term("foo", "A"))), 2),
+                        Occur.SHOULD)
+                    .add(new ConstantScoreQuery(new TermQuery(new Term("foo", "B"))), Occur.SHOULD)
+                    .add(
+                        new BoostQuery(
+                            new ConstantScoreQuery(new TermQuery(new Term("foo", "C"))), 3),
+                        Occur.SHOULD)
+                    .setMinimumNumberShouldMatch(2)
+                    .build());
 
         Scorer scorer =
             searcher
@@ -318,12 +378,13 @@ public class TestWANDScorer extends LuceneTestCase {
         IndexSearcher searcher = newSearcher(reader);
 
         Query query =
-            new BooleanQuery.Builder()
-                .add(new TermQuery(new Term("foo", "A")), Occur.SHOULD)
-                .add(new TermQuery(new Term("foo", "B")), Occur.SHOULD)
-                .add(new TermQuery(new Term("foo", "C")), Occur.SHOULD)
-                .setMinimumNumberShouldMatch(2)
-                .build();
+            new WANDScorerQuery(
+                new BooleanQuery.Builder()
+                    .add(new TermQuery(new Term("foo", "A")), Occur.SHOULD)
+                    .add(new TermQuery(new Term("foo", "B")), Occur.SHOULD)
+                    .add(new TermQuery(new Term("foo", "C")), Occur.SHOULD)
+                    .setMinimumNumberShouldMatch(2)
+                    .build());
 
         Scorer scorer =
             searcher
@@ -365,16 +426,19 @@ public class TestWANDScorer extends LuceneTestCase {
         IndexSearcher searcher = newSearcher(reader);
 
         Query query =
-            new BooleanQuery.Builder()
-                .add(
-                    new BoostQuery(new ConstantScoreQuery(new TermQuery(new Term("foo", "A"))), 2),
-                    Occur.SHOULD)
-                .add(new ConstantScoreQuery(new TermQuery(new Term("foo", "B"))), Occur.SHOULD)
-                .add(
-                    new BoostQuery(new ConstantScoreQuery(new TermQuery(new Term("foo", "C"))), 3),
-                    Occur.SHOULD)
-                .setMinimumNumberShouldMatch(2)
-                .build();
+            new WANDScorerQuery(
+                new BooleanQuery.Builder()
+                    .add(
+                        new BoostQuery(
+                            new ConstantScoreQuery(new TermQuery(new Term("foo", "A"))), 2),
+                        Occur.SHOULD)
+                    .add(new ConstantScoreQuery(new TermQuery(new Term("foo", "B"))), Occur.SHOULD)
+                    .add(
+                        new BoostQuery(
+                            new ConstantScoreQuery(new TermQuery(new Term("foo", "C"))), 3),
+                        Occur.SHOULD)
+                    .setMinimumNumberShouldMatch(2)
+                    .build());
 
         Scorer scorer =
             searcher
@@ -418,20 +482,21 @@ public class TestWANDScorer extends LuceneTestCase {
         Query query =
             new BooleanQuery.Builder()
                 .add(
-                    new BooleanQuery.Builder()
-                        .add(
-                            new BoostQuery(
-                                new ConstantScoreQuery(new TermQuery(new Term("foo", "A"))), 2),
-                            Occur.SHOULD)
-                        .add(
-                            new ConstantScoreQuery(new TermQuery(new Term("foo", "B"))),
-                            Occur.SHOULD)
-                        .add(
-                            new BoostQuery(
-                                new ConstantScoreQuery(new TermQuery(new Term("foo", "D"))), 4),
-                            Occur.SHOULD)
-                        .setMinimumNumberShouldMatch(2)
-                        .build(),
+                    new WANDScorerQuery(
+                        new BooleanQuery.Builder()
+                            .add(
+                                new BoostQuery(
+                                    new ConstantScoreQuery(new TermQuery(new Term("foo", "A"))), 2),
+                                Occur.SHOULD)
+                            .add(
+                                new ConstantScoreQuery(new TermQuery(new Term("foo", "B"))),
+                                Occur.SHOULD)
+                            .add(
+                                new BoostQuery(
+                                    new ConstantScoreQuery(new TermQuery(new Term("foo", "D"))), 4),
+                                Occur.SHOULD)
+                            .setMinimumNumberShouldMatch(2)
+                            .build()),
                     Occur.MUST)
                 .add(new TermQuery(new Term("foo", "C")), Occur.FILTER)
                 .build();
@@ -494,20 +559,21 @@ public class TestWANDScorer extends LuceneTestCase {
         Query query =
             new BooleanQuery.Builder()
                 .add(
-                    new BooleanQuery.Builder()
-                        .add(
-                            new BoostQuery(
-                                new ConstantScoreQuery(new TermQuery(new Term("foo", "A"))), 2),
-                            Occur.SHOULD)
-                        .add(
-                            new ConstantScoreQuery(new TermQuery(new Term("foo", "B"))),
-                            Occur.SHOULD)
-                        .add(
-                            new BoostQuery(
-                                new ConstantScoreQuery(new TermQuery(new Term("foo", "D"))), 4),
-                            Occur.SHOULD)
-                        .setMinimumNumberShouldMatch(2)
-                        .build(),
+                    new WANDScorerQuery(
+                        new BooleanQuery.Builder()
+                            .add(
+                                new BoostQuery(
+                                    new ConstantScoreQuery(new TermQuery(new Term("foo", "A"))), 2),
+                                Occur.SHOULD)
+                            .add(
+                                new ConstantScoreQuery(new TermQuery(new Term("foo", "B"))),
+                                Occur.SHOULD)
+                            .add(
+                                new BoostQuery(
+                                    new ConstantScoreQuery(new TermQuery(new Term("foo", "D"))), 4),
+                                Occur.SHOULD)
+                            .setMinimumNumberShouldMatch(2)
+                            .build()),
                     Occur.MUST)
                 .add(new TermQuery(new Term("foo", "C")), Occur.FILTER)
                 .build();
@@ -553,14 +619,23 @@ public class TestWANDScorer extends LuceneTestCase {
         Query query =
             new BooleanQuery.Builder()
                 .add(
-                    new BoostQuery(new ConstantScoreQuery(new TermQuery(new Term("foo", "A"))), 2),
-                    Occur.SHOULD)
-                .add(new ConstantScoreQuery(new TermQuery(new Term("foo", "B"))), Occur.SHOULD)
+                    new WANDScorerQuery(
+                        new BooleanQuery.Builder()
+                            .add(
+                                new BoostQuery(
+                                    new ConstantScoreQuery(new TermQuery(new Term("foo", "A"))), 2),
+                                Occur.SHOULD)
+                            .add(
+                                new ConstantScoreQuery(new TermQuery(new Term("foo", "B"))),
+                                Occur.SHOULD)
+                            .add(
+                                new BoostQuery(
+                                    new ConstantScoreQuery(new TermQuery(new Term("foo", "D"))), 4),
+                                Occur.SHOULD)
+                            .setMinimumNumberShouldMatch(2)
+                            .build()),
+                    Occur.MUST)
                 .add(new TermQuery(new Term("foo", "C")), Occur.MUST_NOT)
-                .add(
-                    new BoostQuery(new ConstantScoreQuery(new TermQuery(new Term("foo", "D"))), 4),
-                    Occur.SHOULD)
-                .setMinimumNumberShouldMatch(2)
                 .build();
 
         Scorer scorer =
@@ -621,14 +696,23 @@ public class TestWANDScorer extends LuceneTestCase {
         Query query =
             new BooleanQuery.Builder()
                 .add(
-                    new BoostQuery(new ConstantScoreQuery(new TermQuery(new Term("foo", "A"))), 2),
-                    Occur.SHOULD)
-                .add(new ConstantScoreQuery(new TermQuery(new Term("foo", "B"))), Occur.SHOULD)
+                    new WANDScorerQuery(
+                        new BooleanQuery.Builder()
+                            .add(
+                                new BoostQuery(
+                                    new ConstantScoreQuery(new TermQuery(new Term("foo", "A"))), 2),
+                                Occur.SHOULD)
+                            .add(
+                                new ConstantScoreQuery(new TermQuery(new Term("foo", "B"))),
+                                Occur.SHOULD)
+                            .add(
+                                new BoostQuery(
+                                    new ConstantScoreQuery(new TermQuery(new Term("foo", "D"))), 4),
+                                Occur.SHOULD)
+                            .setMinimumNumberShouldMatch(2)
+                            .build()),
+                    Occur.MUST)
                 .add(new TermQuery(new Term("foo", "C")), Occur.MUST_NOT)
-                .add(
-                    new BoostQuery(new ConstantScoreQuery(new TermQuery(new Term("foo", "D"))), 4),
-                    Occur.SHOULD)
-                .setMinimumNumberShouldMatch(2)
                 .build();
 
         Scorer scorer =
@@ -658,24 +742,29 @@ public class TestWANDScorer extends LuceneTestCase {
     }
     IndexReader reader = DirectoryReader.open(w);
     w.close();
-    IndexSearcher searcher = newSearcher(reader);
+    // turn off concurrent search to avoid Random object used across threads resulting into
+    // RuntimeException, as WANDScorerQuery#createWeight has reference to this searcher,
+    // but will be called during searching
+    IndexSearcher searcher = newSearcher(reader, true, true, false);
 
     for (int iter = 0; iter < 100; ++iter) {
       int start = random().nextInt(10);
       int numClauses = random().nextInt(1 << random().nextInt(5));
       BooleanQuery.Builder builder = new BooleanQuery.Builder();
       for (int i = 0; i < numClauses; ++i) {
-        builder.add(maybeWrap(new TermQuery(new Term("foo", Integer.toString(start + i)))), Occur.SHOULD);
+        builder.add(
+            maybeWrap(new TermQuery(new Term("foo", Integer.toString(start + i)))), Occur.SHOULD);
       }
-      Query query = builder.build();
+      Query query = new WANDScorerQuery(builder.build());
 
       CheckHits.checkTopScores(random(), query, searcher);
 
       int filterTerm = random().nextInt(30);
-      Query filteredQuery = new BooleanQuery.Builder()
-          .add(query, Occur.MUST)
-          .add(new TermQuery(new Term("foo", Integer.toString(filterTerm))), Occur.FILTER)
-          .build();
+      Query filteredQuery =
+          new BooleanQuery.Builder()
+              .add(query, Occur.MUST)
+              .add(new TermQuery(new Term("foo", Integer.toString(filterTerm))), Occur.FILTER)
+              .build();
 
       CheckHits.checkTopScores(random(), filteredQuery, searcher);
     }
@@ -683,7 +772,8 @@ public class TestWANDScorer extends LuceneTestCase {
     dir.close();
   }
 
-  public void testRandomWithInfiniteMaxScore() throws IOException {
+  /** Degenerate case: all clauses produce a score of 0. */
+  public void testRandomWithZeroScores() throws IOException {
     Directory dir = newDirectory();
     IndexWriter w = new IndexWriter(dir, newIndexWriterConfig());
     int numDocs = atLeast(1000);
@@ -698,7 +788,70 @@ public class TestWANDScorer extends LuceneTestCase {
     }
     IndexReader reader = DirectoryReader.open(w);
     w.close();
-    IndexSearcher searcher = newSearcher(reader);
+    // turn off concurrent search to avoid Random object used across threads resulting into
+    // RuntimeException, as WANDScorerQuery#createWeight has reference to this searcher,
+    // but will be called during searching
+    IndexSearcher searcher = newSearcher(reader, true, true, false);
+
+    for (int iter = 0; iter < 100; ++iter) {
+      int start = random().nextInt(10);
+      int numClauses = random().nextInt(1 << random().nextInt(5));
+      BooleanQuery.Builder builder = new BooleanQuery.Builder();
+      for (int i = 0; i < numClauses; ++i) {
+        builder.add(
+            maybeWrap(
+                new BoostQuery(
+                    new ConstantScoreQuery(
+                        new TermQuery(new Term("foo", Integer.toString(start + i)))),
+                    0f)),
+            Occur.SHOULD);
+      }
+      Query query = new WANDScorerQuery(builder.build());
+
+      CheckHits.checkTopScores(random(), query, searcher);
+
+      int filterTerm = random().nextInt(30);
+      Query filteredQuery =
+          new BooleanQuery.Builder()
+              .add(query, Occur.MUST)
+              .add(new TermQuery(new Term("foo", Integer.toString(filterTerm))), Occur.FILTER)
+              .build();
+
+      CheckHits.checkTopScores(random(), filteredQuery, searcher);
+    }
+    reader.close();
+    dir.close();
+  }
+
+  /** Test the case when some clauses produce infinite max scores. */
+  public void testRandomWithInfiniteMaxScore() throws IOException {
+    doTestRandomSpecialMaxScore(Float.POSITIVE_INFINITY);
+  }
+
+  /** Test the case when some clauses produce finite max scores, but their sum overflows. */
+  public void testRandomWithMaxScoreOverflow() throws IOException {
+    doTestRandomSpecialMaxScore(Float.MAX_VALUE);
+  }
+
+  private void doTestRandomSpecialMaxScore(float maxScore) throws IOException {
+    Directory dir = newDirectory();
+    IndexWriter w = new IndexWriter(dir, newIndexWriterConfig());
+    int numDocs = atLeast(1000);
+    for (int i = 0; i < numDocs; ++i) {
+      Document doc = new Document();
+      int numValues = random().nextInt(1 << random().nextInt(5));
+      int start = random().nextInt(10);
+      for (int j = 0; j < numValues; ++j) {
+        doc.add(new StringField("foo", Integer.toString(start + j), Store.NO));
+      }
+      w.addDocument(doc);
+    }
+    IndexReader reader = DirectoryReader.open(w);
+    w.close();
+    // turn off concurrent search to avoid Random object used across threads resulting into
+    // RuntimeException, as WANDScorerQuery#createWeight has reference to this searcher,
+    // but will be called during searching
+    IndexSearcher searcher = newSearcher(reader, true, true, false);
 
     for (int iter = 0; iter < 100; ++iter) {
       int start = random().nextInt(10);
@@ -707,19 +860,22 @@ public class TestWANDScorer extends LuceneTestCase {
       for (int i = 0; i < numClauses; ++i) {
         Query query = new TermQuery(new Term("foo", Integer.toString(start + i)));
         if (random().nextBoolean()) {
-          query = new InfiniteMaxScoreWrapperQuery(query, numDocs / TestUtil.nextInt(random(), 1, 100));
+          query =
+              new MaxScoreWrapperQuery(
+                  query, numDocs / TestUtil.nextInt(random(), 1, 100), maxScore);
         }
         builder.add(query, Occur.SHOULD);
       }
-      Query query = builder.build();
+      Query query = new WANDScorerQuery(builder.build());
 
       CheckHits.checkTopScores(random(), query, searcher);
 
       int filterTerm = random().nextInt(30);
-      Query filteredQuery = new BooleanQuery.Builder()
-          .add(query, Occur.MUST)
-          .add(new TermQuery(new Term("foo", Integer.toString(filterTerm))), Occur.FILTER)
-          .build();
+      Query filteredQuery =
+          new BooleanQuery.Builder()
+              .add(query, Occur.MUST)
+              .add(new TermQuery(new Term("foo", Integer.toString(filterTerm))), Occur.FILTER)
+              .build();
 
       CheckHits.checkTopScores(random(), filteredQuery, searcher);
     }
@@ -727,14 +883,16 @@ public class TestWANDScorer extends LuceneTestCase {
     dir.close();
   }
 
-  private static class InfiniteMaxScoreWrapperScorer extends FilterScorer {
+  private static class MaxScoreWrapperScorer extends FilterScorer {
 
     private final int maxRange;
+    private final float maxScore;
     private int lastShallowTarget = -1;
 
-    InfiniteMaxScoreWrapperScorer(Scorer scorer, int maxRange) {
+    MaxScoreWrapperScorer(Scorer scorer, int maxRange, float maxScore) {
       super(scorer);
       this.maxRange = maxRange;
+      this.maxScore = maxScore;
     }
 
     @Override
@@ -746,27 +904,28 @@ public class TestWANDScorer extends LuceneTestCase {
     @Override
     public float getMaxScore(int upTo) throws IOException {
       if (upTo - Math.max(docID(), lastShallowTarget) >= maxRange) {
-        return Float.POSITIVE_INFINITY;
+        return maxScore;
       }
       return in.getMaxScore(upTo);
     }
-
   }
 
-  private static class InfiniteMaxScoreWrapperQuery extends Query {
+  private static class MaxScoreWrapperQuery extends Query {
 
     private final Query query;
     private final int maxRange;
-    
+    private final float maxScore;
+
     /**
-     * If asked for the maximum score over a range of doc IDs that is greater
-     * than or equal to maxRange, this query will return a maximum score of +Infty
+     * If asked for the maximum score over a range of doc IDs that is greater than or equal to
+     * maxRange, this query will return the provided maxScore.
      */
-    InfiniteMaxScoreWrapperQuery(Query query, int maxRange) {
+    MaxScoreWrapperQuery(Query query, int maxRange, float maxScore) {
       this.query = query;
       this.maxRange = maxRange;
+      this.maxScore = maxScore;
     }
-    
+
     @Override
     public String toString(String field) {
       return query.toString(field);
@@ -774,30 +933,37 @@ public class TestWANDScorer extends LuceneTestCase {
 
     @Override
     public boolean equals(Object obj) {
-      return sameClassAs(obj) && query.equals(((InfiniteMaxScoreWrapperQuery) obj).query);
+      if (sameClassAs(obj) == false) {
+        return false;
+      }
+      MaxScoreWrapperQuery that = (MaxScoreWrapperQuery) obj;
+      return query.equals(that.query) && maxRange == that.maxRange && maxScore == that.maxScore;
     }
 
     @Override
     public int hashCode() {
-      return 31 * classHash() + query.hashCode();
+      int hash = classHash();
+      hash = 31 * hash + query.hashCode();
+      hash = 31 * hash + Integer.hashCode(maxRange);
+      hash = 31 * hash + Float.hashCode(maxScore);
+      return hash;
     }
 
     @Override
     public Query rewrite(IndexReader reader) throws IOException {
       Query rewritten = query.rewrite(reader);
       if (rewritten != query) {
-        return new InfiniteMaxScoreWrapperQuery(rewritten, maxRange);
+        return new MaxScoreWrapperQuery(rewritten, maxRange, maxScore);
       }
       return super.rewrite(reader);
     }
 
     @Override
-    public void visit(QueryVisitor visitor) {
-
-    }
+    public void visit(QueryVisitor visitor) {}
 
     @Override
-    public Weight createWeight(IndexSearcher searcher, ScoreMode scoreMode, float boost) throws IOException {
+    public Weight createWeight(IndexSearcher searcher, ScoreMode scoreMode, float boost)
+        throws IOException {
       return new FilterWeight(query.createWeight(searcher, scoreMode, boost)) {
         @Override
         public Scorer scorer(LeafReaderContext context) throws IOException {
@@ -805,7 +971,7 @@ public class TestWANDScorer extends LuceneTestCase {
           if (scorer == null) {
             return null;
           } else {
-            return new InfiniteMaxScoreWrapperScorer(scorer, maxRange);
+            return new MaxScoreWrapperScorer(scorer, maxRange, maxScore);
           }
         }
 
@@ -816,12 +982,12 @@ public class TestWANDScorer extends LuceneTestCase {
             return null;
           } else {
             return new ScorerSupplier() {
-              
+
               @Override
               public Scorer get(long leadCost) throws IOException {
-                return new InfiniteMaxScoreWrapperScorer(supplier.get(leadCost), maxRange);
+                return new MaxScoreWrapperScorer(supplier.get(leadCost), maxRange, maxScore);
               }
-              
+
               @Override
               public long cost() {
                 return supplier.cost();
@@ -833,4 +999,84 @@ public class TestWANDScorer extends LuceneTestCase {
     }
   }
 
+  private static class WANDScorerQuery extends Query {
+    private final BooleanQuery query;
+
+    private WANDScorerQuery(BooleanQuery query) {
+      assert query.clauses().size() == query.getClauses(Occur.SHOULD).size()
+          : "This test utility query is only used to create WANDScorer for disjunctions.";
+      this.query = query;
+    }
+
+    @Override
+    public Weight createWeight(IndexSearcher searcher, ScoreMode scoreMode, float boost)
+        throws IOException {
+      return new Weight(query) {
+
+        @Override
+        public Explanation explain(LeafReaderContext context, int doc) throws IOException {
+          // no-ops
+          return null;
+        }
+
+        @Override
+        public Scorer scorer(LeafReaderContext context) throws IOException {
+          BooleanWeight weight = (BooleanWeight) query.createWeight(searcher, scoreMode, boost);
+          List<Scorer> optionalScorers =
+              weight.weightedClauses.stream()
+                  .map(wc -> wc.weight)
+                  .map(
+                      w -> {
+                        try {
+                          return w.scorerSupplier(context);
+                        } catch (IOException e) {
+                          throw new AssertionError(e);
+                        }
+                      })
+                  .filter(Objects::nonNull)
+                  .map(
+                      ss -> {
+                        try {
+                          return ss.get(Long.MAX_VALUE);
+                        } catch (IOException e) {
+                          throw new AssertionError(e);
+                        }
+                      })
+                  .collect(Collectors.toList());
+
+          if (optionalScorers.size() > 0) {
+            return new WANDScorer(
+                weight, optionalScorers, query.getMinimumNumberShouldMatch(), scoreMode);
+          } else {
+            return weight.scorer(context);
+          }
+        }
+
+        @Override
+        public boolean isCacheable(LeafReaderContext ctx) {
+          return false;
+        }
+      };
+    }
+
+    @Override
+    public String toString(String field) {
+      return "WANDScorerQuery";
+    }
+
+    @Override
+    public void visit(QueryVisitor visitor) {
+      // no-ops
+    }
+
+    @Override
+    public boolean equals(Object other) {
+      return sameClassAs(other) && query.equals(((TestWANDScorer.WANDScorerQuery) other).query);
+    }
+
+    @Override
+    public int hashCode() {
+      return 31 * classHash() + query.hashCode();
+    }
+  }
 }
