@@ -25,7 +25,6 @@ import java.lang.management.ManagementFactory;
 import java.lang.management.ThreadMXBean;
 import java.nio.ByteBuffer;
 import java.nio.ByteOrder;
-import java.nio.FloatBuffer;
 import java.nio.IntBuffer;
 import java.nio.channels.FileChannel;
 import java.nio.file.Files;
@@ -37,15 +36,17 @@ import java.util.HashSet;
 import java.util.Locale;
 import java.util.Objects;
 import java.util.Set;
+import java.util.concurrent.TimeUnit;
 import org.apache.lucene.codecs.KnnVectorsFormat;
 import org.apache.lucene.codecs.KnnVectorsReader;
-import org.apache.lucene.codecs.lucene94.Lucene94Codec;
-import org.apache.lucene.codecs.lucene94.Lucene94HnswVectorsFormat;
-import org.apache.lucene.codecs.lucene94.Lucene94HnswVectorsReader;
+import org.apache.lucene.codecs.lucene95.Lucene95Codec;
+import org.apache.lucene.codecs.lucene95.Lucene95HnswVectorsFormat;
+import org.apache.lucene.codecs.lucene95.Lucene95HnswVectorsReader;
 import org.apache.lucene.codecs.perfield.PerFieldKnnVectorsFormat;
 import org.apache.lucene.document.Document;
 import org.apache.lucene.document.FieldType;
-import org.apache.lucene.document.KnnVectorField;
+import org.apache.lucene.document.KnnByteVectorField;
+import org.apache.lucene.document.KnnFloatVectorField;
 import org.apache.lucene.document.StoredField;
 import org.apache.lucene.index.CodecReader;
 import org.apache.lucene.index.DirectoryReader;
@@ -53,12 +54,13 @@ import org.apache.lucene.index.IndexWriter;
 import org.apache.lucene.index.IndexWriterConfig;
 import org.apache.lucene.index.LeafReader;
 import org.apache.lucene.index.LeafReaderContext;
+import org.apache.lucene.index.StoredFields;
 import org.apache.lucene.index.VectorEncoding;
 import org.apache.lucene.index.VectorSimilarityFunction;
 import org.apache.lucene.search.ConstantScoreScorer;
 import org.apache.lucene.search.ConstantScoreWeight;
 import org.apache.lucene.search.IndexSearcher;
-import org.apache.lucene.search.KnnVectorQuery;
+import org.apache.lucene.search.KnnFloatVectorQuery;
 import org.apache.lucene.search.Query;
 import org.apache.lucene.search.QueryVisitor;
 import org.apache.lucene.search.ScoreDoc;
@@ -69,9 +71,7 @@ import org.apache.lucene.search.Weight;
 import org.apache.lucene.store.Directory;
 import org.apache.lucene.store.FSDirectory;
 import org.apache.lucene.util.BitSetIterator;
-import org.apache.lucene.util.BytesRef;
 import org.apache.lucene.util.FixedBitSet;
-import org.apache.lucene.util.IntroSorter;
 import org.apache.lucene.util.PrintStreamInfoStream;
 import org.apache.lucene.util.SuppressForbidden;
 
@@ -295,7 +295,7 @@ public class KnnGraphTester {
         KnnVectorsReader vectorsReader =
             ((PerFieldKnnVectorsFormat.FieldsReader) ((CodecReader) leafReader).getVectorReader())
                 .getFieldReader(KNN_FIELD);
-        HnswGraph knnValues = ((Lucene94HnswVectorsReader) vectorsReader).getGraph(KNN_FIELD);
+        HnswGraph knnValues = ((Lucene95HnswVectorsReader) vectorsReader).getGraph(KNN_FIELD);
         System.out.printf("Leaf %d has %d documents\n", context.ord, leafReader.maxDoc());
         printGraphFanout(knnValues, leafReader.maxDoc());
       }
@@ -361,7 +361,7 @@ public class KnnGraphTester {
     TopDocs[] results = new TopDocs[numIters];
     long elapsed, totalCpuTime, totalVisited = 0;
     try (FileChannel input = FileChannel.open(queryPath)) {
-      VectorReader targetReader = VectorReader.create(input, dim, vectorEncoding, numIters);
+      VectorReader targetReader = VectorReader.create(input, dim, vectorEncoding);
       if (quiet == false) {
         System.out.println("running " + numIters + " targets; topK=" + topK + ", fanout=" + fanout);
       }
@@ -402,8 +402,10 @@ public class KnnGraphTester {
             }
           }
         }
-        totalCpuTime = (bean.getCurrentThreadCpuTime() - cpuTimeStartNs) / 1_000_000;
-        elapsed = (System.nanoTime() - start) / 1_000_000; // ns -> ms
+        totalCpuTime =
+            TimeUnit.NANOSECONDS.toMillis(bean.getCurrentThreadCpuTime() - cpuTimeStartNs);
+        elapsed = TimeUnit.NANOSECONDS.toMillis(System.nanoTime() - start); // ns -> ms
+        StoredFields storedFields = reader.storedFields();
         for (int i = 0; i < numIters; i++) {
           totalVisited += results[i].totalHits.value;
           for (ScoreDoc doc : results[i].scoreDocs) {
@@ -411,7 +413,7 @@ public class KnnGraphTester {
               // there is a bug somewhere that can result in doc=NO_MORE_DOCS!  I think it happens
               // in some degenerate case (like input query has NaN in it?) that causes no results to
               // be returned from HNSW search?
-              doc.doc = Integer.parseInt(reader.document(doc.doc).get("id"));
+              doc.doc = Integer.parseInt(storedFields.document(doc.doc).get("id"));
             } else {
               System.out.println("NO_MORE_DOCS!");
             }
@@ -469,10 +471,10 @@ public class KnnGraphTester {
   private abstract static class VectorReader {
     final float[] target;
     final ByteBuffer bytes;
+    final FileChannel input;
 
-    static VectorReader create(FileChannel input, int dim, VectorEncoding vectorEncoding, int n)
-        throws IOException {
-      int bufferSize = n * dim * vectorEncoding.byteSize;
+    static VectorReader create(FileChannel input, int dim, VectorEncoding vectorEncoding) {
+      int bufferSize = dim * vectorEncoding.byteSize;
       switch (vectorEncoding) {
         case BYTE:
           return new VectorReaderByte(input, dim, bufferSize);
@@ -482,52 +484,48 @@ public class KnnGraphTester {
       }
     }
 
-    VectorReader(FileChannel input, int dim, int bufferSize) throws IOException {
-      bytes =
-          input.map(FileChannel.MapMode.READ_ONLY, 0, bufferSize).order(ByteOrder.LITTLE_ENDIAN);
+    VectorReader(FileChannel input, int dim, int bufferSize) {
+      this.bytes = ByteBuffer.wrap(new byte[bufferSize]).order(ByteOrder.LITTLE_ENDIAN);
+      this.input = input;
       target = new float[dim];
     }
 
-    void reset() {
+    void reset() throws IOException {
+      input.position(0);
+    }
+
+    protected final void readNext() throws IOException {
+      this.input.read(bytes);
       bytes.position(0);
     }
 
-    abstract float[] next();
+    abstract float[] next() throws IOException;
   }
 
   private static class VectorReaderFloat32 extends VectorReader {
-    private final FloatBuffer floats;
-
-    VectorReaderFloat32(FileChannel input, int dim, int bufferSize) throws IOException {
+    VectorReaderFloat32(FileChannel input, int dim, int bufferSize) {
       super(input, dim, bufferSize);
-      floats = bytes.asFloatBuffer();
     }
 
     @Override
-    void reset() {
-      super.reset();
-      floats.position(0);
-    }
-
-    @Override
-    float[] next() {
-      floats.get(target);
+    float[] next() throws IOException {
+      readNext();
+      bytes.asFloatBuffer().get(target);
       return target;
     }
   }
 
   private static class VectorReaderByte extends VectorReader {
-    private byte[] scratch;
-    private BytesRef bytesRef;
+    private final byte[] scratch;
 
-    VectorReaderByte(FileChannel input, int dim, int bufferSize) throws IOException {
+    VectorReaderByte(FileChannel input, int dim, int bufferSize) {
       super(input, dim, bufferSize);
       scratch = new byte[dim];
-      bytesRef = new BytesRef(scratch);
     }
 
     @Override
-    float[] next() {
+    float[] next() throws IOException {
+      readNext();
       bytes.get(scratch);
       for (int i = 0; i < scratch.length; i++) {
         target[i] = scratch[i];
@@ -535,16 +533,17 @@ public class KnnGraphTester {
       return target;
     }
 
-    BytesRef nextBytes() {
+    byte[] nextBytes() throws IOException {
+      readNext();
       bytes.get(scratch);
-      return bytesRef;
+      return scratch;
     }
   }
 
   private static TopDocs doKnnVectorQuery(
       IndexSearcher searcher, String field, float[] vector, int k, int fanout, Query filter)
       throws IOException {
-    return searcher.search(new KnnVectorQuery(field, vector, k + fanout, filter), k);
+    return searcher.search(new KnnFloatVectorQuery(field, vector, k + fanout, filter), k);
   }
 
   private float checkResults(TopDocs[] results, int[][] nn) {
@@ -663,8 +662,8 @@ public class KnnGraphTester {
     }
     try (FileChannel in = FileChannel.open(docPath);
         FileChannel qIn = FileChannel.open(queryPath)) {
-      VectorReader docReader = VectorReader.create(in, dim, encoding, numDocs);
-      VectorReader queryReader = VectorReader.create(qIn, dim, encoding, numIters);
+      VectorReader docReader = VectorReader.create(in, dim, encoding);
+      VectorReader queryReader = VectorReader.create(qIn, dim, encoding);
       for (int i = 0; i < numIters; i++) {
         float[] query = queryReader.next();
         NeighborQueue queue = new NeighborQueue(topK, false);
@@ -694,10 +693,10 @@ public class KnnGraphTester {
   private int createIndex(Path docsPath, Path indexPath) throws IOException {
     IndexWriterConfig iwc = new IndexWriterConfig().setOpenMode(IndexWriterConfig.OpenMode.CREATE);
     iwc.setCodec(
-        new Lucene94Codec() {
+        new Lucene95Codec() {
           @Override
           public KnnVectorsFormat getKnnVectorsFormatForField(String field) {
-            return new Lucene94HnswVectorsFormat(maxConn, beamWidth);
+            return new Lucene95HnswVectorsFormat(maxConn, beamWidth);
           }
         });
     // iwc.setMergePolicy(NoMergePolicy.INSTANCE);
@@ -705,7 +704,16 @@ public class KnnGraphTester {
     iwc.setUseCompoundFile(false);
     // iwc.setMaxBufferedDocs(10000);
 
-    FieldType fieldType = KnnVectorField.createFieldType(dim, vectorEncoding, similarityFunction);
+    final FieldType fieldType;
+    switch (vectorEncoding) {
+      case BYTE:
+        fieldType = KnnByteVectorField.createFieldType(dim, similarityFunction);
+        break;
+      default:
+      case FLOAT32:
+        fieldType = KnnFloatVectorField.createFieldType(dim, similarityFunction);
+        break;
+    }
     if (quiet == false) {
       iwc.setInfoStream(new PrintStreamInfoStream(System.out));
       System.out.println("creating index in " + indexPath);
@@ -714,18 +722,18 @@ public class KnnGraphTester {
     try (FSDirectory dir = FSDirectory.open(indexPath);
         IndexWriter iw = new IndexWriter(dir, iwc)) {
       try (FileChannel in = FileChannel.open(docsPath)) {
-        VectorReader vectorReader = VectorReader.create(in, dim, vectorEncoding, numDocs);
+        VectorReader vectorReader = VectorReader.create(in, dim, vectorEncoding);
         for (int i = 0; i < numDocs; i++) {
           Document doc = new Document();
           switch (vectorEncoding) {
             case BYTE:
               doc.add(
-                  new KnnVectorField(
+                  new KnnByteVectorField(
                       KNN_FIELD, ((VectorReaderByte) vectorReader).nextBytes(), fieldType));
               break;
             default:
             case FLOAT32:
-              doc.add(new KnnVectorField(KNN_FIELD, vectorReader.next(), fieldType));
+              doc.add(new KnnFloatVectorField(KNN_FIELD, vectorReader.next(), fieldType));
               break;
           }
           doc.add(new StoredField(ID_FIELD, i));
@@ -738,9 +746,10 @@ public class KnnGraphTester {
     }
     long elapsed = System.nanoTime() - start;
     if (quiet == false) {
-      System.out.println("Indexed " + numDocs + " documents in " + elapsed / 1_000_000_000 + "s");
+      System.out.println(
+          "Indexed " + numDocs + " documents in " + TimeUnit.NANOSECONDS.toSeconds(elapsed) + "s");
     }
-    return (int) (elapsed / 1_000_000);
+    return (int) TimeUnit.NANOSECONDS.toMillis(elapsed);
   }
 
   private static void usage() {
@@ -750,40 +759,7 @@ public class KnnGraphTester {
     System.exit(1);
   }
 
-  static class NeighborArraySorter extends IntroSorter {
-    private final int[] node;
-    private final float[] score;
-
-    NeighborArraySorter(NeighborArray neighbors) {
-      node = neighbors.node;
-      score = neighbors.score;
-    }
-
-    int pivot;
-
-    @Override
-    protected void swap(int i, int j) {
-      int tmpNode = node[i];
-      float tmpScore = score[i];
-      node[i] = node[j];
-      score[i] = score[j];
-      node[j] = tmpNode;
-      score[j] = tmpScore;
-    }
-
-    @Override
-    protected void setPivot(int i) {
-      pivot = i;
-    }
-
-    @Override
-    protected int comparePivot(int j) {
-      return Float.compare(score[pivot], score[j]);
-    }
-  }
-
   private static class BitSetQuery extends Query {
-
     private final FixedBitSet docs;
     private final int cardinality;
 
